import { EventEmitter } from "expo-modules-core";
import ExpoHttpServerModule from "./ExpoHttpServerModule";
const emitter = new EventEmitter(ExpoHttpServerModule);
const requestCallbacks = [];
export const start = () => {
    emitter.addListener("onRequest", async (event) => {
        const responseHandler = requestCallbacks.find((c) => c.uuid === event.uuid);
        if (!responseHandler) {
            ExpoHttpServerModule.respond(event.uuid, 404, "Not Found", "application/json", {}, JSON.stringify({ error: "Handler not found" }));
            return;
        }
        const response = await responseHandler.callback(event);
        ExpoHttpServerModule.respond(event.uuid, response.statusCode || 200, response.statusDescription || "OK", response.contentType || "application/json", response.headers || {}, response.body || "{}");
    });
    ExpoHttpServerModule.start();
};
export const route = (path, method, callback) => {
    const uuid = Math.random().toString(16).slice(2);
    requestCallbacks.push({
        method,
        path,
        uuid,
        callback,
    });
    ExpoHttpServerModule.route(path, method, uuid);
};
export const setup = (port, onStatusUpdate) => {
    if (onStatusUpdate) {
        emitter.addListener("onStatusUpdate", async (event) => {
            onStatusUpdate(event);
        });
    }
    ExpoHttpServerModule.setup(port);
};
export const stop = () => ExpoHttpServerModule.stop();
//# sourceMappingURL=index.js.map